//////////////////////////////////////////////////////////////////////////
// Copyright 2001-2011 Aspose Pty Ltd. All Rights Reserved.
//
// This file is part of Aspose.Words. The source code in this file
// is only intended as a supplement to the documentation, and is provided
// "as is", without warranty of any kind, either expressed or implied.
//////////////////////////////////////////////////////////////////////////
package com.aspose.words.demos;

import com.aspose.words.*;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;

public class DemoServlet extends HttpServlet
{
    /**
     * Launched only once, when an application server initializes the servlet.
     */
    public void init() throws ServletException
    {
		// Get the context path of the web application.
        mWebAppDir = this.getServletContext().getRealPath("/");

		// Load the license and init the demo.
		// Rethrow added in order to avoid changing this method's signature.
		try
		{
			Demo.initDemo();

			mGeneratedDir = new File(mWebAppDir, GENERATED_DIR_SHORT_NAME);
			if (!mGeneratedDir.exists())
				mGeneratedDir.mkdir();

			// *** Licensing: Put the license file into the /WEB-INF folder,
			// otherwise Aspose.Words will work in evaluation mode.
			loadLicense();
		}
		catch (Exception e)
		{
			throw new ServletException(e);
		}
    }

    /**
     * Attemps to load the Aspose.Words license.
     */
	private void loadLicense() throws Exception
	{
		// The servlet container prohibits an external access to the WEB-INF folder.
		File licenseFile = new File(mWebAppDir, "WEB-INF/Aspose.Words.Java.lic");
		if (licenseFile.exists())
		{
			// This shows how to license Aspose.Words.
			// If you don't specify a license, Aspose.Words works in evaluation mode.
			License license = new License();
			license.setLicense(licenseFile.getAbsolutePath());
		}
	}

    /**
     * Creates and returns the demo selected by the user.
     */
    private Demo createDemo(String demoName)
    {
		if ("AlternatingRows".equals(demoName))
			return new AlternatingRowsDemo();
		else if ("DinnerInvitation".equals(demoName))
			return new DinnerInvitationDemo();
		else if ("DocumentBuilder".equals(demoName))
			return new DocumentBuilderDemo();
		else if ("EmployeesReport".equals(demoName))
			return new EmployeesReportDemo();
		else if ("MailingLabels".equals(demoName))
			return new MailingLabelsDemo();
		else if ("ProductCatalog".equals(demoName))
			return new ProductCatalogDemo();
		else if ("SalesInvoice".equals(demoName))
			return new SalesInvoiceDemo();
		else if ("StampAndWatermark".equals(demoName))
			return new StampAndWatermarkDemo();
        else if ("Barcode".equals(demoName))
			return new BarCodeDemo();
		else
			throw new RuntimeException("Unknown demo selected.");
    }

    /**
    * Called from the web-app index page (because the POST method is chosen for the input form).
    */
    protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
    {
        // Get the demo selected by the user.
        String demoName = request.getParameter("demo");

        // Get the output format selected by the user.
        String formatType = request.getParameter("format");

        // If null, do the default demo.
        if (demoName == null)
            demoName = "DinnerInvitation";

        // Create and execute the demo, it produces the document in memory.
        Demo demo = createDemo(demoName);

        // Rethrow in order to avoid changing this method's signature.
        try
        {
            Document doc = demo.execute();

            // Once we have a document, we can save it to a file, stream or send to the client's browser.
            // We just send the document to the browser in this case.
            boolean openNewWindow = request.getParameter("openNewWindow") != null;
            sendToBrowser(doc, demoName, formatType, openNewWindow, response);
            response.flushBuffer();
        }
        catch (Exception e)
        {
            throw new RuntimeException(e);
        }
    }

    /**
     * Sends the document to the client's browser.
     */
    private void sendToBrowser(Document doc, String demoName, String formatType, boolean openNewWindow, HttpServletResponse response)
            throws Exception
    {
        String extension = formatType;

        // Change the extension for these formats manually.
        if(formatType.equals("WML") || formatType.equals("FOPC"))
            extension = "XML";

        String fileName = demoName + "." + extension;

		// Add the Response header
        if(openNewWindow)
            response.setHeader("content-disposition","attachment; filename=" + fileName);
        else
            response.addHeader("content-disposition","inline; filename=" + fileName);

        if("DOC".equals(formatType))
        {
            // Set the content type
            response.setContentType("application/msword");

            // Save the generated document to the Response.
            doc.save(response.getOutputStream(), SaveFormat.DOC);
        }
		else if("DOCX".equals(formatType))
		{
	        response.setContentType("application/vnd.openxmlformats-officedocument.wordprocessingml.document");
			doc.save(response.getOutputStream(), SaveFormat.DOCX);
		}
        else if("PDF".equals(formatType))
		{
	        response.setContentType("application/pdf");
			doc.save(response.getOutputStream(), SaveFormat.PDF);
		}
        else if("RTF".equals(formatType))
		{
	        response.setContentType("text/rtf");
			doc.save(response.getOutputStream(), SaveFormat.RTF);
		}
        else if("HTML".equals(formatType))
        {
			sendToBrowserAsHtml(doc, response, demoName);
		}
        else if("TXT".equals(formatType))
        {
            response.setContentType("text/plain");
            doc.save(response.getOutputStream(), SaveFormat.TEXT);
        }
        else if("MHTML".equals(formatType))
		{
            // Set options used in HTML export.
            HtmlSaveOptions htmlOptions = new HtmlSaveOptions(SaveFormat.MHTML);

            // We don't need to export headers and footers in HTML format.
            htmlOptions.setExportHeadersFootersMode(ExportHeadersFootersMode.NONE);
            // Output the XHTML header.
            htmlOptions.setExportXhtmlTransitional(true);

	        response.setContentType("message/rfc822");
			doc.save(response.getOutputStream(), htmlOptions);
		}
        else if("XPS".equals(formatType))
		{
	        response.setContentType("application/vnd.ms-xpsdocument");
			doc.save(response.getOutputStream(), SaveFormat.XPS);
		}
        else if("ODT".equals(formatType))
		{
	        response.setContentType("application/vnd.oasis.opendocument.text");
			doc.save(response.getOutputStream(), SaveFormat.ODT);
		}
        else if("EPUB".equals(formatType))
		{
            // Set options used in HTML export.
            HtmlSaveOptions htmlOptions = new HtmlSaveOptions(SaveFormat.EPUB);

            // We don't need to export headers and footers in HTML format.
            htmlOptions.setExportHeadersFootersMode(ExportHeadersFootersMode.NONE);
            // Output the XHTML header.
            htmlOptions.setExportXhtmlTransitional(true);

	        response.setContentType("application/xhtml+xml");
			doc.save(response.getOutputStream(), htmlOptions);
		}
        else if("XML".equals(formatType))
		{
	        response.setContentType("application/vnd.openxmlformats-officedocument.wordprocessingml.document");
			doc.save(response.getOutputStream(), SaveFormat.WORD_ML);
		}
        else if("FOPC".equals(formatType))
		{
	        response.setContentType("application/vnd.openxmlformats-officedocument.wordprocessingml.document");
			doc.save(response.getOutputStream(), SaveFormat.FLAT_OPC);
		}
        else if("SWF".equals(formatType))
		{
	        response.setContentType("application/x-shockwave-flash");
			doc.save(response.getOutputStream(), SaveFormat.SWF);
		}
    }

	/**
	 * Stream the document to the client browser in HTML format.
	 *
	 * Option 1: Save the document to a file and redirect this file
	 * (along with image files saved in the same folder) to the client browser.
	 */
	private void sendToBrowserAsHtml(Document doc, HttpServletResponse response, String fileName) throws Exception
	{
        HtmlSaveOptions htmlOptions = new HtmlSaveOptions(SaveFormat.HTML);

        // We don't need to export headers and footers in HTML format.
		htmlOptions.setExportHeadersFootersMode(ExportHeadersFootersMode.NONE);
        // Output the XHTML header.
		htmlOptions.setExportXhtmlTransitional(true);

		// Save the document and the document's images into the "generated" directory.
		String relativeUrl = GENERATED_DIR_SHORT_NAME + "/" + fileName;
		String realPath = this.getServletContext().getRealPath(relativeUrl);
		doc.save(realPath, htmlOptions);

		// Redirecting to the HTML document rather than saving it into the response stream
		// makes sure the browser can find the images in the same folder as the document.
		response.setContentType("text/html");
		response.sendRedirect(relativeUrl);
	}

	/**
	 * Stream the document to the client browser in HTML format.
	 *
	 * Option 2: Save the document into the response stream.
	 * Illustrates how to use ExportImagesFolder and HtmlExportImagesFolderAlias properties.
	 *
	 * Note that Aspose.Words generates a unique name for each image file (when saving the document
	 * in HTML format to a stream) so you will need extra logic to remove these generated files
	 * or alternatively save them into a temporary directory and clear it periodically.
	 */
	private void sendToBrowserAsHtmlResponse(Document doc, HttpServletResponse response) throws Exception
	{
		response.setContentType("text/html");

        HtmlSaveOptions htmlOptions = new HtmlSaveOptions(SaveFormat.HTML);

        // We don't need to export headers and footers to HTML format.
		htmlOptions.setExportHeadersFootersMode(ExportHeadersFootersMode.NONE);
        // Output the XHTML header.
		htmlOptions.setExportXhtmlTransitional(true);

        // Image paths for the server (local) - Aspose.Words puts generated image files here.
        htmlOptions.setImagesFolder(mGeneratedDir.getPath());
        // Image paths for the browser (external) - Aspose.Words uses this to calculate external
		// paths to the generated images and puts them inside the generated html page.
        htmlOptions.setImagesFolderAlias(GENERATED_DIR_SHORT_NAME);

		doc.save(response.getOutputStream(), htmlOptions);
	}

	private String mWebAppDir;
    private File mGeneratedDir;
    private static final String GENERATED_DIR_SHORT_NAME = "generated";
}

