//////////////////////////////////////////////////////////////////////////
// Copyright 2001-2011 Aspose Pty Ltd. All Rights Reserved.
//
// This file is part of Aspose.Words. The source code in this file
// is only intended as a supplement to the documentation, and is provided
// "as is", without warranty of any kind, either expressed or implied.
//////////////////////////////////////////////////////////////////////////
package com.aspose.words.demos;


import com.aspose.words.*;
import java.io.File;
import java.net.*;
import java.sql.*;
import java.util.ArrayList;

/**
 * Base class for Aspose.Words.Demos classes that execute specific demos.
 */
public abstract class Demo
{
	/**
     * Called before launching any concrete demo-classes.
	 * We need the application path so we can find the database and template documents.
     */
	public static void initDemo() throws Exception
	{
		// Calculate common project dirs and files used by all demos.
		mProjectDir = getProjectDir();
		File baseDir = mProjectDir.getParentFile();
		mDocumentsDir = new File(baseDir, "Documents");
		mGeneratedDir = new File(mProjectDir, "generated");
		mDatabaseDir = new File(baseDir, "Database");
		mDatabase = new File(mDatabaseDir, "Northwind.mdb");
		mResultSetCollection = new ArrayList();

		// Check all paths exist.
		checkExistense();

		// Clear the directory for generated files.
		if (mGeneratedDir.exists())
		{
			File[] files = mGeneratedDir.listFiles();
			for (int i = 0; i < files.length; i++)
				files[i].delete();
		}
		else
		{
			mGeneratedDir.mkdir();
		}

		// Create a connection to the database.
		createConnection();
	}

	/**
	 * Calculates the root directory of the demo project.
	 *
	 * The demo project directory is "demos/Aspose.Words.Demos".
	 * By default compiled classes will be placed by compiler into "classes" subdirectory
	 * of the project directory, so Demo.class (this) will be placed into
	 * "demos/Aspose.Words.Demos/classes/com/aspose/words/demos/Demo.class".
	 */
	private static File getProjectDir() throws Exception
	{
		URI uri = Demo.class.getResource("Demo.class").toURI();

		String path = new File(uri.getPath()).getAbsolutePath();
		String subPath = "\\classes\\com\\aspose\\words\\demos\\Demo.class";

		return new File(path.substring(0, path.length() - subPath.length()));
	}

	/**
	 * Note: The wrong project path will generally result in the wrong path to dependent directories and the
     * generated files.
	 */
	private static void checkExistense()
	{
		if (!mProjectDir.exists())
			throw new IllegalArgumentException("Can't find the Project dir: " + mProjectDir.getAbsolutePath());

		if (!mDocumentsDir.exists())
			throw new IllegalArgumentException("Can't find the template documents dir: " + mDocumentsDir.getAbsolutePath());

		if (!mDatabaseDir.exists())
			throw new IllegalArgumentException("Can't find the database dir: " + mDatabaseDir.getAbsolutePath());

		if (!mDatabase.exists())
			throw new IllegalArgumentException("Can't find the database file: " + mDatabase.getAbsolutePath());
	}

	/**
	 * Utility function that creates a database statement
	 */
    public static Statement createStatement() throws Exception
	{
		return mConnection.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
	}

    /**
	 * Cleans up and closes all ResultSets and Statements used. Then closes the connection to the database.
	 */
	public static void closeResources() throws Exception
	{
		// Iterate through each ResultSet used by the demos. Close the result set and
		// parent statement.
		for(int i = 0; i < mResultSetCollection.size(); i++)
		{
			ResultSet rs = (ResultSet) mResultSetCollection.get(i);
		    if(rs != null)
			{
				Statement statement = rs.getStatement();

				rs.close();

				if(statement != null)
					statement.close();
			}
		}

		mConnection.close();
	}

    /**
	 * Utility function that creates a connection to the Database.
	 */
    public static void createConnection() throws Exception
	{
		//  Load a DB driver that is used by the demos
		Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");
		// Compose connection string.
		String connectionString = "jdbc:odbc:DRIVER={Microsoft Access Driver (*.mdb)};" +
								  "DBQ=" + mDatabase + ";UID=Admin";
		// DSN-less DB connection.
		mConnection = DriverManager.getConnection(connectionString);
	}

	/**
     * Called by concrete demo-class. Calculates paths to files used solely by this concrete demo.
     */
    protected void init(String demoName)
	{
		// Calculate files used solely by this concrete demo.
		mDemoName = demoName;
		File docTemplate = new File(mDocumentsDir, mDemoName + ".doc");
		File docxTemplate = new File(mDocumentsDir, mDemoName + ".docx");

		// Check to see if a DOC or DOCX template is used else throw an exception if no template
		// could be found.
		if (docTemplate.exists())
			mTemplate = docTemplate;
		else if (docxTemplate.exists())
			mTemplate = docxTemplate;
		else
			throw new IllegalArgumentException("Can't find the template document file for demo: " + demoName);
	}

    /**
     * Executes the demo and returns a generated document. Implement in the derived classes.
     */
    public abstract Document execute() throws Exception;

	/**
	 * Executes a query to the demo database using a new statement and returns the result in a ResultSet.
	 */
	protected ResultSet executeQuery(String query) throws Exception
	{
		ResultSet rs = createStatement().executeQuery(query);
		// Add this resultset to be closed later.
		mResultSetCollection.add(rs);
		return rs;
	}

    /**
	* Sets the output format for this demo
	*/
	public void setOutputFormat(String format)
	{
		mOutputFormat = format;
	}

    /**
	* Gets the output format for this demo
	*/
	public String getOutputFormat()
	{
		return mOutputFormat;
	}

    /**
     * Gets path to the image to be inserted into the document
     */
    protected String getImagePath(){
         return new File(mDocumentsDir, "Aspose.Words.gif").getPath();
    }

    /**
	 * Return the demo name without the extension.
	 */
	protected String getDemoNameWithoutExtension()
	{
        if(mDemoName.indexOf(".") >= 0)
            return mDemoName.substring(0, mDemoName.lastIndexOf("."));
		else
            return mDemoName;
	}

	/**
	 * Returns an demo name.
	 */
	public String getDemoName()
	{
		return mDemoName;
	}

	/**
	 * Returns a full path where the template document used by this demos is stored.
	 */
	public String getPathToTemplateDoc()
	{
		return mTemplate.getAbsolutePath();
	}

	/**
	 * Returns a full path where the document generated by this demo should be stored.
	 */
	public String getPathToGeneratedDoc()
	{
		return new File(mGeneratedDir, getDemoNameWithoutExtension() + ".doc").getAbsolutePath();
	}

	/**
	 * Returns a full path where the document generated by this demo should be stored.
	 */
	public String getPathToGeneratedHtml()
	{
		return new File(mGeneratedDir, getDemoNameWithoutExtension() + ".html").getAbsolutePath();
	}

	/**
	 * Returns a full path where the document generated by this demo should be stored.
	 */
	public String getPathToGeneratedDocx()
	{
		return new File(mGeneratedDir, getDemoNameWithoutExtension() + ".docx").getAbsolutePath();
	}

	/**
	 * Returns a full path where the document generated by this demo should be stored.
	 */
	public String getPathToGeneratedPdf()
	{
		return new File(mGeneratedDir, getDemoNameWithoutExtension() + ".pdf").getAbsolutePath();
	}

	/**
	 * Returns a full path where the document generated by this demo should be stored.
	 */
	public String getPathToGeneratedTxt()
	{
		return new File(mGeneratedDir, getDemoNameWithoutExtension() + ".txt").getAbsolutePath();
	}

	/**
	 * Returns a full path where the document generated by this demo should be stored.
	 */
	public String getPathToGeneratedXps()
	{
		return new File(mGeneratedDir, getDemoNameWithoutExtension() + ".xps").getAbsolutePath();
	}

	/**
	 * Returns a full path where the document generated by this demo should be stored.
	 */
	public String getPathToGeneratedOdt()
	{
		return new File(mGeneratedDir, getDemoNameWithoutExtension() + ".odt").getAbsolutePath();
	}

	/**
	 * Returns a full path where the document generated by this demo should be stored.
	 */
	public String getPathToGeneratedMhtml()
	{
		return new File(mGeneratedDir, getDemoNameWithoutExtension() + ".mhtml").getAbsolutePath();
	}

	/**
	 * Returns a full path where the document generated by this demo should be stored.
	 */
	public String getPathToGeneratedRtf()
	{
		return new File(mGeneratedDir, getDemoNameWithoutExtension() + ".rtf").getAbsolutePath();
	}

	/**
	 * Returns a full path where the document generated by this demo should be stored.
	 */
	public String getPathToGeneratedWml()
	{
		return new File(mGeneratedDir, getDemoNameWithoutExtension() + ".xml").getAbsolutePath();
	}

	/**
	 * Returns a full path where the document generated by this demo should be stored.
	 */
	public String getPathToGeneratedFopc()
	{
		return new File(mGeneratedDir, getDemoNameWithoutExtension() + ".fopc").getAbsolutePath();
	}

	/**
	 * Returns a full path where the document generated by this demo should be stored.
	 */
	public String getPathToGeneratedEpub()
	{
		return new File(mGeneratedDir, getDemoNameWithoutExtension() + ".epub").getAbsolutePath();
	}

	/**
	 * Returns a full path where the document generated by this demo should be stored.
	 */
	public String getPathToGeneratedSwf()
	{
		return new File(mGeneratedDir, getDemoNameWithoutExtension() + ".swf").getAbsolutePath();
	}

	/**
	 * Returns a root dir of this Aspose.Words release.
	 * (the dir where unzipped downloaded Aspose.Words distributive).
	 * Probably in that dir will be placed license file.
	 *
	 * Note: Can be called only after the demo initialization (initDemo() method).
	 */
	static File getRootDistributiveDir()
	{
		return mProjectDir.getParentFile().getParentFile();
	}

	/**
	 * Project dir. e.g (Aspose.Words.Java installation path)\\demos\\Aspose.Words.Demos
	 */
	protected static File mProjectDir;
	/**
	 * Stores a connection to the database.
	 */
	protected static Connection mConnection;
	/**
	 * Stores a list of ResultSet objects used in the demos. These are released when the demos have finished.
	 */
	protected static ArrayList mResultSetCollection;
	/**
	 * Directory for template documents used by the demos: demos\\Documents.
	 */
	protected static File mDocumentsDir;
	/**
	 * Directory for documents generated by the demos: demos\\generated.
	 */
	protected static File mGeneratedDir;
	/**
	 * Directory for the Database used by the demos: demos\\Database.
	 */
	protected static File mDatabaseDir;
	/**
	 * Database file used by the demos: demos\\Database\\Northwind.mdb.
	 */
	protected static File mDatabase;
	/**
	 * The name of this demo. Template document and all generated documents use this name.
	 */
	protected String mDemoName;
	/**
	 * Template document file used by the concrete demo.
	 */
	protected File mTemplate;
    /**
	 * The output format that this demo will be saved as. Save as Word .doc format by default.
	 */
	protected String mOutputFormat = "doc";
}
