//////////////////////////////////////////////////////////////////////////
// Copyright 2001-2011 Aspose Pty Ltd. All Rights Reserved.
//
// This file is part of Aspose.Words. The source code in this file
// is only intended as a supplement to the documentation, and is provided
// "as is", without warranty of any kind, either expressed or implied.
//////////////////////////////////////////////////////////////////////////
package com.aspose.words.demos;

import com.aspose.words.*;

import javax.imageio.ImageIO;
import java.awt.*;
import java.io.File;

/**
 * Manipulates a document programmatically. Inserts formatted text, paragraphs,
 * images, tables and other content into a Word document.
 */
public class DocumentBuilderDemo extends Demo
{
	public DocumentBuilderDemo()
	{
		init("DocumentBuilderDemo");
	}

    public Document execute() throws Exception
    {
        // Open the template document
		Document doc = new Document(getPathToTemplateDoc());

        // Once the builder is created, its cursor is positioned at the beginning of the document.
        DocumentBuilder builder = new DocumentBuilder(doc);

        // Specify font formatting before adding text.
        builder.getFont().setSize(16);
        builder.getFont().setBold(true);
        builder.getFont().setColor(Color.BLUE);
        builder.getFont().setName("Arial");
        builder.setUnderline(Underline.DASH);
        builder.write("This text");
        builder.setUnderline(Underline.NONE);
        builder.write(" is inserted at the beginning of the document and demonstrates ");

        // Set shading and border around a run of text, then clear the border.
        builder.getFont().getShading().setForegroundPatternColor(Color.CYAN);
        builder.getFont().getShading().setTexture(TextureIndex.TEXTURE_SOLID);
        builder.write("various font");
        builder.getFont().getShading().clearFormatting();
        builder.getFont().getBorder().setLineStyle(LineStyle.SINGLE);
        builder.write(" formatting ");
        builder.getFont().getBorder().clearFormatting();
        builder.writeln("options.");
        builder.insertParagraph();

        // Move the insertion cursor to a merge field.
        builder.moveToMergeField("MyInsertionPlace1");

        // Specify paragraph formatting before calling InsertParagraph or Writeln.
        builder.getParagraphFormat().getBorders().get(BorderType.TOP).setLineStyle(LineStyle.SINGLE);
        builder.getParagraphFormat().getBorders().get(BorderType.TOP).setLineWidth(1.5);
        builder.getParagraphFormat().setAlignment(ParagraphAlignment.CENTER);
        builder.getParagraphFormat().getShading().setForegroundPatternColor(new Color(167, 191, 222));
        builder.getParagraphFormat().getShading().setTexture(TextureIndex.TEXTURE_SOLID);
        builder.getParagraphFormat().setLeftIndent(ConvertUtil.inchToPoint(0.3));
        builder.getParagraphFormat().setSpaceBefore(12);
        builder.getParagraphFormat().setSpaceAfter(12);
        builder.getFont().setName("Arial");
        builder.getFont().setSize(12);
        builder.write("This fragment demonstrates use of paragraph formatting to specify ");
        builder.writeln("alignment, spacing, borders and shading.");
        builder.getParagraphFormat().clearFormatting();

        // Create a fancy looking table inside the document.
        buildChessBoard(builder);

        // Insert another section and demonstrate changing page setup properties.
        builder.getFont().setSize(12);
        builder.insertBreak(BreakType.SECTION_BREAK_NEW_PAGE);
        builder.getPageSetup().setPaperSize(PaperSize.A5);
        builder.getPageSetup().setOrientation(Orientation.LANDSCAPE);
        builder.writeln("This text is in a section with different page size and orientation.");

		// Import an HTML snippet which is converted into document content using insertHtml.
		builder.insertHtml("<p>This text is <B><I>Bold Italic</B></I> inserted from html formatted string.");

        // Insert an image from disk into the document
        builder.getParagraphFormat().setAlignment(ParagraphAlignment.CENTER);
        builder.writeln();
        builder.writeln("Insert an image in the document here.");
        builder.insertImage(ImageIO.read(new File(super.getImagePath())));

        // Move the cursor to a position in the header.
        builder.moveToMergeField("MyHeader");
        builder.getFont().clearFormatting();
        builder.getFont().setItalic(true);
        builder.write("This text is inserted into the header.");

        // Remove any left over merge fields from the document as they are not needed anymore.
        doc.getMailMerge().deleteFields();

		return doc;
    }

    /**
     * Creates a chess like table to test borders and shading.
     */
    public void buildChessBoard(DocumentBuilder builder) throws Exception
    {
        // Reset to default font and paragraph formatting.
        builder.getFont().clearFormatting();
        builder.getParagraphFormat().clearFormatting();

        builder.getParagraphFormat().setAlignment(ParagraphAlignment.CENTER);
        builder.insertParagraph();
        builder.writeln("This shows how to build a table in a document.");

        builder.getFont().setSize(6);
        builder.getFont().setName("Tahoma");

        // It is possible to change row format before (like here) and also during creation of the row.
        builder.getRowFormat().setHeight(ConvertUtil.inchToPoint(0.4));	// 1 inch = 72 points.
        builder.getRowFormat().setHeightRule(HeightRule.EXACTLY);

        // It is possible to change cell format before (like here) and also during creation of the cell.
        builder.getCellFormat().setWidth(ConvertUtil.inchToPoint(0.4));
        builder.getCellFormat().setVerticalAlignment(CellVerticalAlignment.CENTER);

        BorderCollection borders = builder.getCellFormat().getBorders();
        borders.setLineStyle(LineStyle.SINGLE);
        borders.setLineWidth(2);
        borders.setColor(Color.BLUE);

        Table table = builder.startTable();

        for (int y = 0; y < 8; y++)
        {
            for (int x = 0; x < 8; x++)
            {
                builder.insertCell();
                builder.write("X" + x + ", Y" + y);

                // This alternates cell backgrounds.
                boolean isBlack = (((x + y) & 0x01) != 0);
                builder.getCellFormat().getShading().setBackgroundPatternColor(
                    (isBlack) ? new Color(167, 191, 222) : new Color(211, 223, 238));
            }
            builder.endRow();
        }

        table.setAlignment(TableAlignment.CENTER);
        builder.endTable();
    }
}
