//////////////////////////////////////////////////////////////////////////
// Copyright 2001-2011 Aspose Pty Ltd. All Rights Reserved.
//
// This file is part of Aspose.Words. The source code in this file
// is only intended as a supplement to the documentation, and is provided
// "as is", without warranty of any kind, either expressed or implied.
//////////////////////////////////////////////////////////////////////////

package com.aspose.words.demos;

import com.aspose.words.*;

import java.io.File;

/**
 * Creates a list of products grouped by product category. Shows how to create reports with parent-child
 * data, combine documents, move sections between documents and mail merge images stored in files outside
 * of the database.
 */
public class ProductCatalogDemo extends Demo
{
	public ProductCatalogDemo()
	{
		init("ProductCatalogDemo");
	}

	public Document execute() throws Exception
	{
		// Open the template document
		Document doc = new Document(getPathToTemplateDoc());

        // Product catalog is an example of parent-child report (Category is the parent and Product is the child)
        // and calls for use of nested mail merge regions. This can be done easily using Aspose.Words with just a few lines of code
        // as shown below.

        // The document template to be merged contains the StartTable and EndTable tags of a region nested within another region.
        // The merge is achieved by first populating a dataset with each datatable and passing the entire dataset to the mail merge engine. Aspose.Words matches
        // the region names in the template to the table names and identifies the relationships between the regions using DataSet.Relations.

        // Populate the Dataset with the parent data and the data from the child table
        DataSet dataSet = getDataSet();

        // Handle the insertion of images during mail merge
        doc.getMailMerge().setFieldMergingCallback(new HandleImageMerging());

        // Merge the data with the document template
        doc.getMailMerge().executeWithRegions(dataSet);

        // Add front and back cover pages to the report.
        // This demonstrates a general approach of creating documents  that is possible
        // by moving, copying or removing document sections.
        Document coverDoc = new Document(new File(mDocumentsDir, "ProductCatalogDemoCover.doc").getAbsolutePath());

        // Here we import the required sections and add them to the destination document.
        Node frontPage = doc.importNode(coverDoc.getFirstSection(), true);
        doc.prependChild(frontPage);

        Node orderPage = doc.importNode(coverDoc.getLastSection(), true);
        doc.appendChild(orderPage);

        return doc;
	}

	/**
	 * A callback called for every Image:[FieldName] merge field in the document.
	 */
	public class HandleImageMerging implements IFieldMergingCallback
	{
		public void fieldMerging(FieldMergingArgs args)
		{
			// Do Nothing
		}
		public void imageFieldMerging(ImageFieldMergingArgs args)
		{
			if ("PictureFileName".equals(args.getFieldName()))
			{
				// You can do a number of things in this event handler, this is just one scenario.
				// In our case the field in the database contains a file name (without path information)
				// of the image of the product category.
				String shortFileName = (String) args.getFieldValue();
				// We know that the image files are stored in the same folder as the database and
				// so we create the full file name.
				// Here, we pass the image file name to Aspose.Words mail merge engine.
				// Alternatively, we could have passed an Image Object or a Stream by
				// args.setImage(BufferedImage) or args.setImageStream(InputStream) setters.
				args.setImageFileName( new File(mDatabaseDir, shortFileName).getAbsolutePath());
			}
		}
	}

    private DataSet getDataSet()  throws Exception
	{
		// Note that mail merging using DataSet or DataTable classes is like working with disconnected data. All data needed for the mail merge
		// operation must be loaded into memory and must be scrollable. Either the ResultSets objects must be scrollable and open
		// for the duration of mail merge or the data can be loaded in a CachedRowSet.
        DataSet dataSet = new DataSet();

        DataTable categories = new DataTable(executeQuery("Select TOP 5 * from Categories"), "Categories");
        dataSet.getTables().add(categories);

		DataTable orderDetails = new DataTable(executeQuery("SELECT * from Products"), "Products");
		dataSet.getTables().add(orderDetails);

		// In order for nested mail merge to work, the mail merge engine must know the relation between parent and child tables.
		// Add a DataRelation to specify relations between these tables.
		dataSet.getRelations().add(new DataRelation(
				"CategoriesToProducts",
				"Categories",
				"Products",
				new String[]{"CategoryID"},
				new String[]{"CategoryID"}));

        return dataSet;
	}

}
