//////////////////////////////////////////////////////////////////////////
// Copyright 2001-2011 Aspose Pty Ltd. All Rights Reserved.
//
// This file is part of Aspose.Words. The source code in this file
// is only intended as a supplement to the documentation, and is provided
// "as is", without warranty of any kind, either expressed or implied.
//////////////////////////////////////////////////////////////////////////
package com.aspose.words.demos;

import com.aspose.cells.*;
import com.aspose.words.Document;
import com.aspose.words.*;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.sql.ResultSet;

/**
 * Demonstrates integration with Aspose.Cells. Inserts a chart into a document at a location specified
 * by a bookmark.
 */
public class SalesReportDemo extends Demo
{
	public SalesReportDemo()
	{
		init("SalesReportDemo");
	}

    public Document execute() throws Exception
    {
        // Open the template document
		Document doc = new Document(getPathToTemplateDoc());

		// Create DocumentBuilder so we can modify the document.
        DocumentBuilder builder = new DocumentBuilder(doc);

        // There is already a bookmark in the document where we want the chart inserted, navigate to it.
        builder.moveToBookmark("SalesByCategory");

        // Create the chart image and insert it into the document.
        BufferedImage salesByCategoryImage = createSalesByCategoryChart();
        builder.insertImage(salesByCategoryImage);

		return doc;
	}

    /**
     *  Creates and returns a chart for the demo using Aspose.Cells.
     */
    private BufferedImage createSalesByCategoryChart() throws Exception
    {
        // This crosstab report has Year in the first column, followed by product categories in other columns.
        // Each row contains total sales per year per product category.
        ResultSet rs = executeQuery("SELECT * FROM AsposeWordSalesReport_Crosstab");

        // Initialize Workbook
        Workbook workbook = new Workbook();

        // Initialize worksheet
        Worksheet sheet = workbook.getWorksheets().getSheet(0);

        // Set the name of worksheet
        sheet.setName("Data");

        // Initialize Cells
        Cells cells = sheet.getCells();

        // Import data from the ResultSet and create a stacked area chart from it.
        cells.importResultSet(rs, "A1", true);

        Chart chart = sheet.getCharts().addChart(ChartType.AREA_STACKED, 1, 1, 20, 10);

        // Set some visual options.
        chart.getPlotArea().getArea().setForegroundColor(Color.WHITE);
        chart.getCategoryAxis().getMajorGridLines().setVisible(true);
        chart.getValueAxis().getMajorGridLines().setVisible(true);
        // Set currency formatting for the value axis.
        chart.getValueAxis().setNumberFormat("$#,##0;$-#,##0");
        chart.getChartArea().getBorder().setVisible(false);
        chart.getCategoryAxis().setBetweenCategories(false);
        chart.getCategoryAxis().setMinorTickMark(TickMarkerType.OUTSIDE);
        chart.getValueAxis().setMinorTickMark(TickMarkerType.OUTSIDE);

        // Set input data ranges.
        chart.getNSeries().add("Data!B2:I3", true);
        chart.getNSeries().setCategoryData("Data!A2:A3");

        // Assign each series name from title cell values.
        for (int i = 0; i < chart.getNSeries().size(); i++)
        {
            chart.getNSeries().get(i).setName(cells.getCell(0, i + 1).getValue().toString());
        }

		// Render the chart to image.
		ByteArrayOutputStream out = new ByteArrayOutputStream();
        chart.toImage(out, new ImageOptions());

		byte[] data = out.toByteArray();
		BufferedImage image = ImageIO.read(new ByteArrayInputStream(data));

		// Return the image.
		return image;
    }
}